/*
 *  Copyright (c) 2016, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.wso2.carbon.launcher.extensions;

import org.wso2.carbon.launcher.CarbonServerEvent;
import org.wso2.carbon.launcher.CarbonServerListener;
import org.wso2.carbon.launcher.Constants;
import org.wso2.carbon.launcher.extensions.model.BundleInfo;
import org.wso2.carbon.launcher.utils.Utils;

import java.io.IOException;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.List;
import java.util.Optional;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * This class deploys the OSGi bundles in CARBON_HOME/{@value org.wso2.carbon.launcher.Constants#OSGI_LIB} directory in
 * the Carbon Server.
 * <p>
 * For this purpose, the OSGi bundle information retrieved from the {@value org.wso2.carbon.launcher.Constants#OSGI_LIB}
 * directory bundles are
 * updated in the bundles.info file of each and every, existing Carbon runtime, along with the bundle startup
 * information of each bundle.
 *
 * @since 5.0.0
 */
public class OSGiLibBundleDeployer implements CarbonServerListener {
    private static final Logger logger = Logger.getLogger(OSGiLibBundleDeployer.class.getName());

    @Override
    public void notify(CarbonServerEvent event) {
        if (event.getType() == CarbonServerEvent.STARTING) {
            Path carbonHome = Utils.getCarbonHomeDirectory();
            Path libDirectoryPath = Paths.get(carbonHome.toString(), Constants.OSGI_LIB);
            String profile = Optional.ofNullable(System.getProperty(Constants.PROFILE))
                    .orElse(Constants.DEFAULT_PROFILE);

            try {
                logger.log(Level.FINE,
                        "Loading the new OSGi bundle information from " + Constants.OSGI_LIB + " folder...");
                List<BundleInfo> newBundlesInfo = OSGiLibBundleDeployerUtils.getBundlesInfo(libDirectoryPath);
                logger.log(Level.FINE, "Successfully loaded the new OSGi bundle information from " +
                        Constants.OSGI_LIB + " folder");

                OSGiLibBundleDeployerUtils.updateOSGiLib(carbonHome.toString(), profile, newBundlesInfo);
            } catch (IOException e) {
                logger.log(Level.SEVERE, "Failed to update the OSGi bundle information of Carbon Runtime: " + profile,
                        e);
            }
        }
    }
}
